"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fc = require("fast-check");
var R = require("fp-ts/lib/Record");
var t = require("io-ts");
var objectTypes = ['ExactType', 'InterfaceType', 'PartialType'];
function getArbitrary(codec) {
    var type = codec;
    switch (type._tag) {
        case 'StringType':
            return fc.string();
        case 'UndefinedType':
            return fc.constant(undefined);
        case 'NumberType':
            return fc.float();
        case 'BooleanType':
            return fc.boolean();
        case 'LiteralType':
            return fc.constant(type.value);
        case 'InterfaceType':
            return fc.record(R.record.map(type.props, getArbitrary));
        case 'PartialType':
            return fc.record(R.record.map(type.props, function (type) { return getArbitrary(t.union([t.undefined, type])); }));
        case 'UnionType':
            return fc.oneof.apply(fc, type.types.map(getArbitrary));
        case 'KeyofType':
            return fc.oneof.apply(fc, Object.keys(type.keys).map(fc.constant));
        case 'NullType':
            return fc.constant(null);
        case 'TupleType':
            return fc.tuple.apply(fc, type.types.map(getArbitrary));
        case 'UnknownType':
            return fc.anything();
        case 'VoidType':
            return fc.constant(undefined);
        case 'ArrayType':
            return fc.array(getArbitrary(type.type));
        case 'ExactType':
            return getArbitrary(type.type);
        case 'RefinementType':
            return getArbitrary(type.type).filter(type.predicate);
        case 'IntersectionType':
            var isObjectIntersection = objectTypes.includes(type.types[0]._tag);
            return isObjectIntersection
                ? fc.tuple.apply(fc, type.types.map(getArbitrary)).map(function (values) { return Object.assign.apply(Object, [{}].concat(values)); })
                    .filter(type.is)
                : fc.oneof.apply(fc, type.types.map(getArbitrary)).filter(type.is);
    }
    throw new Error("Codec not supported: " + codec);
}
exports.getArbitrary = getArbitrary;
